@@ -240,6 +240,14 @@ impl BackendMailbox for MboxMailbox {
     fn count(&self) -> Result<(usize, usize)> {
         Ok((*self.unseen.lock()?, *self.total.lock()?))
     }
+
+    fn as_any(&self) -> &dyn std::any::Any {
+        self
+    }
+
+    fn as_any_mut(&mut self) -> &mut dyn std::any::Any {
+        self
+    }
 }
 
 /// `BackendOp` implementor for Mbox
@@ -730,7 +738,7 @@ pub struct MessageIterator<'a> {
     pub format: Option<MboxFormat>,
 }
 
-impl<'a> Iterator for MessageIterator<'a> {
+impl Iterator for MessageIterator<'_> {
     type Item = Result<Envelope>;
     fn next(&mut self) -> Option<Self::Item> {
         if self.input.is_empty() {
